//
// See the file LICENSE for redistribution information.
//
// Copyright (c) 2002-2004
//	Sleepycat Software.  All rights reserved.
//
// $Id: XmlQueryExpression.cpp,v 1.24 2004/09/20 21:43:29 gmf Exp $
//

#include "dbxml_config.h"
#include "dbxml/XmlPortability.hpp"
#include "dbxml/XmlQueryExpression.hpp"
#include "dbxml/XmlNamespace.hpp"
#include "dbxml/XmlResults.hpp"
#include "dbxml/XmlException.hpp"
#include "dbxml/XmlValue.hpp"
#include "dbxml/XmlTransaction.hpp"
#include "QueryExpression.hpp"

using namespace DbXml;

XmlQueryExpression::XmlQueryExpression(QueryExpression *expression)
	: expression_(expression)
{
	expression_->acquire();
}

XmlQueryExpression::~XmlQueryExpression()
{
	expression_->release();
}

XmlQueryExpression::XmlQueryExpression(const XmlQueryExpression& o)
	: expression_(o.expression_)
{
	if (expression_ != 0)
		expression_->acquire();
}

XmlQueryExpression &XmlQueryExpression::operator=(const XmlQueryExpression& o)
{
	if (this != &o && expression_ != o.expression_) {
		if (expression_ != 0)
			expression_->release();
		expression_ = o.expression_;
		if (expression_ != 0)
			expression_->acquire();
	}
	return *this;
}

const std::string &XmlQueryExpression::getQuery() const
{
	return expression_->getQuery();
}

std::string XmlQueryExpression::getQueryPlan() const
{
	return expression_->getQueryPlan();
}

XmlResults XmlQueryExpression::execute(XmlQueryContext &context, u_int32_t flags) const
{
	if(flags & ~(DB_DIRTY_READ|DB_RMW|DBXML_LAZY_DOCS)) {
		throw XmlException(XmlException::INVALID_VALUE, "Invalid flags to method XmlQueryExpression::execute");
	}

	try {
		return expression_->execute(0, 0, context, flags);
	} catch (DbException &e) {
		throw XmlException(e);
	}
}

XmlResults XmlQueryExpression::execute(XmlTransaction &txn, XmlQueryContext &context, u_int32_t flags) const
{
	if(flags & ~(DB_DIRTY_READ|DB_RMW|DBXML_LAZY_DOCS)) {
		throw XmlException(XmlException::INVALID_VALUE, "Invalid flags to method XmlQueryExpression::execute");
	}

	try {
		return expression_->execute(txn, 0, context, flags);
	} catch (DbException &e) {
		throw XmlException(e);
	}
}

XmlResults XmlQueryExpression::execute(const XmlValue &contextItem, XmlQueryContext &context, u_int32_t flags) const
{
	if(flags & ~(DB_DIRTY_READ|DB_RMW|DBXML_LAZY_DOCS)) {
		throw XmlException(XmlException::INVALID_VALUE, "Invalid flags to method XmlQueryExpression::execute");
	}

	try {
		return expression_->execute(0, contextItem, context, flags);
	} catch (DbException &e) {
		throw XmlException(e);
	}
}

XmlResults XmlQueryExpression::execute(XmlTransaction &txn, const XmlValue &contextItem, XmlQueryContext &context, u_int32_t flags) const
{
	if(flags & ~(DB_DIRTY_READ|DB_RMW|DBXML_LAZY_DOCS)) {
		throw XmlException(XmlException::INVALID_VALUE, "Invalid flags to method XmlQueryExpression::execute");
	}

	try {
		return expression_->execute(txn, contextItem, context, flags);
	} catch (DbException &e) {
		throw XmlException(e);
	}
}
