//
// See the file LICENSE for redistribution information.
//
// Copyright (c) 2002-2004
//	Sleepycat Software.  All rights reserved.
//
// $Id: XmlResults.cpp,v 1.33 2004/10/20 16:57:38 jsnelson Exp $
//

#include "dbxml_config.h"
#include "dbxml/XmlPortability.hpp"
#include "dbxml/XmlException.hpp"
#include "dbxml/XmlResults.hpp"
#include "dbxml/XmlDocument.hpp"
#include "Results.hpp"

using namespace DbXml;

XmlResults::XmlResults(Results *results)
  : results_(results)
{
  results_->acquire();
}

XmlResults::~XmlResults()
{
	results_->release();
}

XmlResults::XmlResults(const XmlResults &o)
	: results_(o.results_)
{
	if (results_ != 0)
		results_->acquire();
}

XmlResults &XmlResults::operator=(const XmlResults &o)
{
	if (this != &o && results_ != o.results_) {
		if (results_ != 0)
			results_->release();
		results_ = o.results_;
		if (results_ != 0)
			results_->acquire();
	}
	return *this;
}

void XmlResults::reset()
{
	results_->reset();
}

size_t XmlResults::size() const
{
	return results_->size();
}

bool XmlResults::hasNext()
{
	return results_->hasNext();
}

bool XmlResults::hasPrevious()
{
	return results_->hasPrevious();
}

bool XmlResults::next(XmlValue &value)
{
	int err = 0;
	try {
		err = results_->next(value);
	} catch (DbException &e) {
		throw XmlException(e);
	}
	if (err != 0)
		throw XmlException(err);

	return (!value.isNull());
}

bool XmlResults::peek(XmlValue &value)
{
	int err = 0;
	try {
		err = results_->peek(value);
	} catch (DbException &e) {
		throw XmlException(e);
	}
	if (err != 0)
		throw XmlException(err);

	return (!value.isNull());
}

bool XmlResults::previous(XmlValue &value)
{
	int err = 0;
	try {
		err = results_->previous(value);
	} catch (DbException &e) {
		throw XmlException(e);
	}
	if (err != 0)
		throw XmlException(err);

	return (!value.isNull());
}

bool XmlResults::next(XmlDocument &document)
{
	XmlValue value;
	if(next(value)) {
		document = value.asDocument();
		return true;
	}
	return false;
}

bool XmlResults::previous(XmlDocument &document)
{
	XmlValue value;
	if(previous(value)) {
		document = value.asDocument();
		return true;
	}
	return false;
}

bool XmlResults::peek(XmlDocument &document)
{
	XmlValue value;
	if(peek(value)) {
		document = value.asDocument();
		return true;
	}
	return false;
}

void XmlResults::add(const XmlValue &value)
{
	results_->add(value);
}

