//
// See the file LICENSE for redistribution information.
//
// Copyright (c) 2002-2004
//	Sleepycat Software.  All rights reserved.
//
// $Id: LazyCommand.cpp,v 1.2 2004/08/19 16:52:27 gareth Exp $
//

#include "LazyCommand.hpp"

using namespace DbXml;
using namespace std;

string LazyCommand::getCommandName() const
{
  return "lazy";
}

string LazyCommand::getBriefHelp() const
{
  return "Sets lazy evaluation on or off in the default context";
}

string LazyCommand::getMoreHelp() const
{
  return
    string("Takes either on or off as an argument. This command uses the\n") +
    string("XmlQueryContext::setEvaluationType() method.");
}

void LazyCommand::execute(Args &args, Environment &env)
{
  if(args.size() != 2) {
    throw CommandException("Wrong number of arguments");
  }

  if(args[1] == "on") {
    env.context().setEvaluationType(XmlQueryContext::Lazy);
    if(env.verbose()) cout << "Lazy evaluation on" << endl;
  }
  else {
    env.context().setEvaluationType(XmlQueryContext::Eager);
    if(env.verbose()) cout << "Lazy evaluation off" << endl;
  }
}

