# See the file LICENSE for redistribution information.
#
# Copyright (c) 2000-2004
#	Sleepycat Software.  All rights reserved.
#
# $Id: batch_query.tcl,v 1.3 2004/10/14 16:19:50 awood Exp $

# Batch execution of the container querying (benchmarking)
# Use comments / adjust lists to configure what is done!
#
# Invocation
#   tclsh batch_load ?
#   tclsh batch_load <dbenv> <data>
#
# Arguments
#   <dbenv>  - path to DB XML environment directory
#   <path>   - path to directory that contains utility executables
#

proc usage { name } {
    puts "Usage: $name <dbenv> <data>"
    puts "\t<dbenv> - path to DB XML environment"
    puts "\t<path> - path to directory containing DB XML utility executables"
}

# help option
if { $argc == 1 && [lindex $argv 0] == "?"} {
    usage $argv0
    exit
}

set err 1

# verify arguments
if { $argc != 2 } {
    puts "Incorrect number of arguments: $argc"
    usage $argv0
    exit $err
}

set dbenv [lindex $argv 0]
set exe_path [lindex $argv 1]

# load the procedures 
set xbench_tcl [file join [file dirname $argv0] xbench.tcl]
source $xbench_tcl

# verify the dbenv
if {[file exists $dbenv] != 1} {
    puts "Cannot find the DB XML environment directory \"$dbenv\""
    exit $err
}
set ret [catch { glob $dbenv/*.dbxml } result]
if { $ret != 0 } {
    puts "There are no *.dbxml files in the DB XML environment directory \"$dbenv\""
    exit $err
}

# verify the path
if {[file exists [file join $exe_path "query_runner"] ] != 1} {
    puts "Cannot find query_runner in \"$exe_path\""
    exit $err
}

# the test data is assumed to be in the same directory as this script
set test_data_path [file dirname $argv0]

################################################################
# Container querying
#  modify the lists to control what is loaded!

puts "Querying containers..."

#set sizes [list "miniscule" "tiny" "small" "medium" "normal" "large"]
set sizes [list "tiny" "small" "medium"]
foreach size $sizes {
    puts "\tSize \"$size\""
    set err [query_container $dbenv $test_data_path $exe_path $size]
    if { $err != 0 } {
        puts "Error while querying container"
        exit $err
    }
}

set err 0
exit $err
